@@ -5,18 +5,21 @@
 (defparameter *autoconf-prefix* "/usr/local")
 (defparameter *autoconf-exec_prefix* "/usr/local")
 (defparameter *autoconf-package* "maxima")
-(defparameter *autoconf-version* "5.38.1")
+(defparameter *autoconf-version* "5.39.0")
 (defparameter *autoconf-libdir* "/usr/local/lib")
 (defparameter *autoconf-libexecdir* "/usr/local/libexec")
 (defparameter *autoconf-datadir* "/usr/local/share")
 (defparameter *autoconf-infodir* "/usr/local/info")
-(defparameter *autoconf-host* "powerpc-unknown-openbsd6.0")
+(defparameter *autoconf-host* "alpha-unknown-openbsd6.1")
 ;; This variable is kept for backwards compatibiliy reasons:
 ;; We seem to be in the fortunate position that we sometimes need to check for windows.
 ;; But at least until dec 2015 we didn't need to check for a specific windows flavour.
 (defparameter *autoconf-win32* "false")
 (defparameter *autoconf-windows* "false")
 (defparameter *autoconf-ld-flags* "")
+
+;; This will be T if this was a lisp-only build
+(defparameter *autoconf-lisp-only-build* (eq t '@lisp_only_build@))
  
-(defparameter *maxima-source-root* "/usr/obj/ports/maxima-5.38.1/maxima-5.38.1")
+(defparameter *maxima-source-root* "/usr/obj/ports/maxima-5.39.0/maxima-5.39.0")
 (defparameter *maxima-default-layout-autotools* "true")
